/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger.json.action;

import java.util.UUID;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONValue;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.config.ConfigTools;
import com.openexchange.config.cascade.ComposedConfigProperty;
import com.openexchange.config.cascade.ConfigView;
import com.openexchange.config.cascade.ConfigViewFactory;
import com.openexchange.exception.OXException;
import com.openexchange.java.Strings;
import com.openexchange.java.util.UUIDs;
import com.openexchange.messenger.MessengerAPI;
import com.openexchange.messenger.MessengerConfiguration;
import com.openexchange.messenger.MessengerExceptionCodes;
import com.openexchange.messenger.exception.MessengerException;
import com.openexchange.messenger.json.MessengerRequest;
import com.openexchange.messenger.session.StaticAuthSession;
import com.openexchange.server.ServiceLookup;
import com.openexchange.tools.session.ServerSession;


/**
 * {@link CreateGuestUser}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public class CreateGuestUser extends MessengerAction {

    /**
     * Initializes a new {@link CreateGuestUser}.
     * @param services
     */
    public CreateGuestUser(ServiceLookup services) {
        super(services);
    }

    @Override
    protected AJAXRequestResult perform(final MessengerRequest messengerRequest) throws OXException, JSONException, MessengerException {
        ServerSession session = messengerRequest.getSession();
        int userId = session.getUserId();
        int contextId = session.getContextId();

        MessengerConfiguration configuration = MessengerConfiguration.getInstance();

        // Check if allowed
        {
            boolean enabled;
            ConfigViewFactory factory = getService(ConfigViewFactory.class);
            if (null == factory) {
                enabled = configuration.isGuestModeEnabled();
            } else {
                ConfigView view = factory.getView(userId, contextId);
                ComposedConfigProperty<Boolean> property = view.property("com.openexchange.messenger.guestMode.enabled", boolean.class);
                if (property.isDefined()) {
                    enabled = property.get().booleanValue();
                } else {
                    enabled = configuration.isGuestModeEnabled();
                }
            }
            if (false == enabled) {
                throw MessengerExceptionCodes.GUEST_INVITATION_DENIED.create(Integer.valueOf(userId), Integer.valueOf(contextId));
            }
        }

        // The invitee's E-Mail address
        String emailAddress = messengerRequest.checkParameter("mail");
        String firstName = messengerRequest.checkParameter("firstName");
        String lastName = messengerRequest.checkParameter("lastName");

        // The TTL
        long expirationStamp;
        {
            int ttlSeconds;
            String sTtl = messengerRequest.getParameter("ttl");
            if (Strings.isEmpty(sTtl)) {
                ttlSeconds = configuration.getGuestModeTTLSeconds();
            } else {
                int tmp;
                try {
                    tmp = ConfigTools.parseTimespanSecs(sTtl.trim());
                } catch (final Exception e) {
                    tmp = ConfigTools.parseTimespanSecs("5D");
                }
                ttlSeconds = tmp;
            }
            expirationStamp = System.currentTimeMillis() + (ttlSeconds * 1000L);
        }

        // Generate token
        UUID token = UUID.randomUUID();

        // Expires
        String expiration = MessengerAPI.formatExpirationDate(expirationStamp);

        // pass to end point
        JSONValue result;
        String userName;
        {
            MessengerAPI<StaticAuthSession> api = new MessengerAPI<StaticAuthSession>(new StaticAuthSession(configuration));
            userName = new StringBuilder(32).append("guest-").append(UUIDs.getUnformattedString(UUID.randomUUID())).append('@').append(getContextDomain(messengerRequest.getRequestData(), session)).toString();
            try {
                result = api.createGuestUser(userId, contextId, userName, emailAddress, firstName, lastName, token, expiration, null);
            } catch (final MessengerException e) {
                throw e;
            }
        }

        // Determine call-back URL
        String serverUrl;
        {
            String callbackUrl = configuration.getGuestModeCallbackUrl();
            serverUrl = Strings.isEmpty(callbackUrl) ? messengerRequest.getRequestData().constructURL(null, false).toString() : callbackUrl;
        }

        // Return result
        return new AJAXRequestResult(new JSONObject(8).put("token", UUIDs.getUnformattedString(token)).put("userName", userName).put("emailAddress", emailAddress).put("firstName", firstName).put("lastName", lastName).put("expiresAt", expiration).put("url", serverUrl), "json");
    }

    private String getContextDomain(AJAXRequestData requestData, ServerSession session) {
        String login = session.getLogin();
        if (Strings.isEmpty(login)) {
            return sanitize(requestData.getHostname());
        }
        int pos = login.indexOf('@');
        return sanitize(pos > 0 ? login.substring(pos + 1) : requestData.getHostname());
    }

    private String sanitize(String domain) {
        int len = domain.length();
        StringBuilder sb = new StringBuilder(len);
        for (int i = 0; i < len; i++) {
            char c = domain.charAt(i);
            if (c == '_') {
                sb.append('-');
            } else if (Strings.isWhitespace(c)) {
                sb.append('-');
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    @Override
    public String getAction() {
        return "create_guest_user";
    }

}
