/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2020 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger.provisioning.interceptor;

import java.util.Map;
import org.slf4j.Logger;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.container.Contact;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.ldap.User;
import com.openexchange.messenger.MessengerAPI;
import com.openexchange.messenger.MessengerExceptionCodes;
import com.openexchange.messenger.exception.MessengerException;
import com.openexchange.messenger.exception.MessengerUnlinkedException;
import com.openexchange.messenger.provisioning.AbstractMessengerAccess;
import com.openexchange.messenger.session.StaticAuthSession;
import com.openexchange.server.ServiceLookup;
import com.openexchange.user.UserServiceInterceptor;

/**
 * {@link MessengerUserServiceInterceptor}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public class MessengerUserServiceInterceptor extends AbstractMessengerAccess implements UserServiceInterceptor {

    private static final Logger LOGGER = org.slf4j.LoggerFactory.getLogger(MessengerUserServiceInterceptor.class);

    private static abstract class SafeInvocable<V> {

        public SafeInvocable() {
            super();
        }

        public V invoke(final Context context, final User user, final Contact contactData, final MessengerAPI<StaticAuthSession> connector) {
            try {
                return doInvoke(context, user, contactData, connector);
            } catch (final OXException e) {
                logError(e, LOGGER);
            } catch (final RuntimeException e) {
                logError(e, LOGGER);
            }
            return null;
        }

        protected abstract V doInvoke(Context context, User user, Contact contactData, MessengerAPI<StaticAuthSession> connector) throws OXException;

        protected abstract void logError(final Exception e, final Logger logger);
    }

    private final ServiceLookup services;

    /**
     * Initializes a new {@link MessengerUserServiceInterceptor}.
     */
    public MessengerUserServiceInterceptor(final ServiceLookup services) {
        super();
        this.services = services;
    }

    @Override
    public int getRanking() {
        return 0;
    }

    @Override
    public void beforeCreate(final Context context, final User user, final Contact contactData) throws OXException {
        // Nothing
    }

    @Override
    public void afterCreate(final Context context, final User user, final Contact contactData) {
        new SafeInvocable<Void>() {

            @Override
            protected Void doInvoke(Context context, User user, Contact contactData, MessengerAPI<StaticAuthSession> connector) throws OXException {
                try {
                    connector.createUser(context, user, contactData);
                } catch (final MessengerUnlinkedException e) {
                    throw MessengerExceptionCodes.UNLINKED_ERROR.create();
                } catch (final MessengerException e) {
                    throw MessengerExceptionCodes.MESSENGER_ERROR.create(e, e.getMessage());
                } catch (final RuntimeException e) {
                    throw MessengerExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
                }
                return null;
            }

            @Override
            protected void logError(final Exception e, final Logger logger) {
                logger.error("Failed to create user {} in context {} at messenger end point.", user.getId(), context.getContextId(), e);
            }
        }.invoke(context, user, contactData, getConnector());
    }

    @Override
    public void beforeUpdate(final Context context, final User user, final Contact contactData, final Map<String, Object> properties) throws OXException {
        // Nothing
    }

    @Override
    public void afterUpdate(final Context context, final User user, final Contact contactData, final Map<String, Object> properties) throws OXException {
        // nothing
        /*
         * try { // Check if either user acquired or lost "messenger" capability final CapabilityService capabilityService =
         * services.getOptionalService(CapabilityService.class); if (null != capabilityService) { final boolean hasMessengerEnabled =
         * capabilityService.getCapabilities(user.getId(), contactData.getContextId(), false, false).contains("messenger"); if
         * (hasMessengerEnabled) { getConnector().createUser(user, contactData); } else { getConnector().deleteUser(user, contactData); } }
         * } catch (final OXMessengerUnlinkedException e) { throw OxMessengerExceptionCodes.UNLINKED_ERROR.create(); } catch (final OXMessengerException e) {
         * throw OxMessengerExceptionCodes.MESSENGER_ERROR.create(e, e.getMessage()); }
         */
    }

    @Override
    public void beforeDelete(final Context context, final User user, final Contact contactData) {
        new SafeInvocable<Void>() {

            @Override
            protected Void doInvoke(Context context, User user, Contact contactData, MessengerAPI<StaticAuthSession> connector) throws OXException {
                try {
                    connector.deleteUser(context, user, contactData);
                } catch (final MessengerUnlinkedException e) {
                    throw MessengerExceptionCodes.UNLINKED_ERROR.create();
                } catch (final MessengerException e) {
                    throw MessengerExceptionCodes.MESSENGER_ERROR.create(e, e.getMessage());
                } catch (final RuntimeException e) {
                    throw MessengerExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
                }
                return null;
            }

            @Override
            protected void logError(final Exception e, final Logger logger) {
                logger.error("Failed to delete user {} in context {} at messenger end point.", user.getId(), context.getContextId(), e);
            }
        }.invoke(context, user, contactData, getConnector());
    }

    @Override
    public void afterDelete(final Context context, final User user, final Contact contactData) throws OXException {
        // Nothing
    }

}
