/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2020 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONValue;
import com.openexchange.groupware.container.Contact;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.java.Strings;
import com.openexchange.java.util.UUIDs;
import com.openexchange.messenger.exception.MessengerApiException;
import com.openexchange.messenger.exception.MessengerException;
import com.openexchange.messenger.exception.MessengerUnlinkedException;
import com.openexchange.messenger.rest.RESTUtility;
import com.openexchange.messenger.session.Session;
import com.openexchange.messenger.session.StaticAuthSession;

/**
 * {@link MessengerAPI} - The Messenger API.
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public class MessengerAPI<S extends Session> {

    /**
     * Gets a new API instance initialized with the static configuration.
     *
     * @return A new API instance based on static configuration
     */
    public static MessengerAPI<StaticAuthSession> getStaticMessengerAPI() {
        return new MessengerAPI<StaticAuthSession>(new StaticAuthSession(MessengerConfiguration.getInstance()));
    }

    // ---------------------------------------------------------------------------------------------------------------- //

    /**
     * The version of the API that this code uses.
     */
    public static final int VERSION = 1;

    /** The session */
    private final S session;

    /**
     * Initializes a new {@link MessengerAPI}.
     *
     * @param session The associated session
     */
    public MessengerAPI(final S session) {
        super();
        this.session = session;
    }

    /**
     * Gets the associated session.
     */
    public S getSession() {
        return session;
    }

    /**
     * Throws a {@link MessengerUnlinkedException} if the session in this instance is not linked.
     */
    protected void assertAuthenticated() throws MessengerUnlinkedException {
        if (!session.isLinked()) {
            throw new MessengerUnlinkedException();
        }
    }

    /**
     * Checks if specified user exists.
     *
     * @param userId The user identifier
     * @param contextId The context identifier
     * @throws MessengerException If operation fails
     */
    public boolean existsUser(final int userId, final int contextId) throws MessengerException {
        assertAuthenticated();

        try {
            final Map<String, Object> params = new LinkedHashMap<String, Object>(8);
            params.put("applicationUserId", Integer.valueOf(userId));
            params.put("applicationTenantId", Integer.valueOf(contextId));

            final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            final JSONObject response =
                RESTUtility.requestJsonRpc(configuration.getAPIServer(), configuration.getAPIKey(), "provisioning.checkUserExists", params, session).toObject();

            checkForError(response);

            final JSONObject jResult = response.getJSONObject("result");
            return jResult.getBoolean("exists");
        } catch (final JSONException e) {
            throw new MessengerException(e);
        }
    }

    private static final SimpleDateFormat DATE_FORMAT_EXPIRES_AT;
    static {
        final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd' 'HH:mm:ss", Locale.US);
        DATE_FORMAT_EXPIRES_AT = sdf;
    }

    /**
     * Formats given <tt>'expiresAt'</tt> date; e.g. <code>"2014-05-02 14:27:10"</code>
     *
     * @param expirationStamp The expiration UTC time stamp
     * @return The formatted date
     */
    public static String formatExpirationDate(final long expirationStamp) {
        synchronized (DATE_FORMAT_EXPIRES_AT) {
            return DATE_FORMAT_EXPIRES_AT.format(new Date(expirationStamp));
        }
    }

    /**
     * Creates a guest user at VW end point.
     *
     * <pre>
     * createGuestUser:
     * {
     *       "jsonrpc": "2.0",
     *       "method": "provisioning.createGuestUser",
     *       "params": {
     *                     "apiKey": "API_KEY_01",
     *                     "applicationUserId": 4,
     *                     "applicationTenantId": 1,
     *                     "userName": "guest-129386123612783632@tenantdomain.tld",
     *                     "firstName": "Firstname",
     *                     "lastName": "Lastname",
     *                     "email": "username@emaildomain.tld",
     *                     "token": "HBKXv1Xh54wTC6m9423bwl3789OFL078",
     *                     "expiresAt": "2014-05-02 13:30:00"
     *        },
     *        "id": 102
     * }
     * </pre>
     *
     * @param userId The user identifier
     * @param contextId The context identifier
     * @param userName A unique random user name
     * @param emailAddress The E-Mail address of the guest
     * @param token The identity token
     * @param expirationStamp The UTC time stamp when the invitation/guest account terminates
     * @param optHost The optional host
     * @return The URL to end point
     * @throws MessengerException yet to be defined
     */
    public JSONValue createGuestUser(int userId, int contextId, String userName, String emailAddress, String firstName, String lastName, UUID token, long expirationStamp, String optHost) throws MessengerException {
        return createGuestUser(userId, contextId, userName, emailAddress, firstName, lastName, token, formatExpirationDate(expirationStamp), optHost);
    }

    /**
     * Creates a guest user at VW end point.
     *
     * <pre>
     * createGuestUser:
     * {
     *       "jsonrpc": "2.0",
     *       "method": "provisioning.createGuestUser",
     *       "params": {
     *                     "apiKey": "API_KEY_01",
     *                     "applicationUserId": 4,
     *                     "applicationTenantId": 1,
     *                     "userName": "guest-129386123612783632@tenantdomain.tld",
     *                     "firstName": "Firstname",
     *                     "lastName": "Lastname",
     *                     "email": "username@emaildomain.tld",
     *                     "token": "HBKXv1Xh54wTC6m9423bwl3789OFL078",
     *                     "expiresAt": "2014-05-02 13:30:00"
     *        },
     *        "id": 102
     * }
     * </pre>
     *
     * @param userId The user identifier
     * @param contextId The context identifier
     * @param userName A unique random user name
     * @param emailAddress The E-Mail address of the guest
     * @param name The name of the guest
     * @param token The identity token
     * @param expiration The expiration when the invitation/guest account terminates; see {@link #formatExpirationDate(long)}
     * @param optHost The optional host
     * @return The URL to end point
     * @throws MessengerException yet to be defined
     */
    public JSONValue createGuestUser(int userId, int contextId, String userName, String emailAddress, String firstName, String lastName, UUID token, String expiration, String optHost) throws MessengerException {
        assertAuthenticated();

        try {
            final Map<String, Object> params = new LinkedHashMap<String, Object>(10);
            params.put("applicationUserId", Integer.valueOf(userId));
            params.put("applicationTenantId", Integer.valueOf(contextId));
            params.put("userName", userName);
            params.put("firstName", firstName);
            params.put("lastName", lastName);
            params.put("email", emailAddress);
            params.put("token", UUIDs.getUnformattedString(token));
            params.put("expiresAt", expiration);

            final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            final String host = Strings.isEmpty(optHost) ? configuration.getAPIServer() : optHost;
            final JSONValue jResponse = RESTUtility.requestJsonRpc(host, configuration.getAPIKey(), "provisioning.createGuestUser", params, session);

            checkForError(jResponse);

            return (JSONValue) jResponse.toObject().get("result");
        } catch (final JSONException e) {
            throw new MessengerException(e.getMessage(), e);
        }
    }

    /**
     * Deletes a guest user at VW end point.
     *
     * @param userId The user identifier
     * @param contextId The context identifier
     * @param optHost The optional host
     * @throws MessengerException yet to be defined
     */
    public void deleteGuestUser(final int userId, final int contextId, final String emailAddress, final String optHost) throws MessengerException {
        assertAuthenticated();

        final Map<String, Object> params = new LinkedHashMap<String, Object>(8);
        params.put("applicationUserId", Integer.valueOf(userId));
        params.put("applicationTenantId", Integer.valueOf(contextId));
        params.put("userName", emailAddress);

        final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
        final String host = Strings.isEmpty(optHost) ? configuration.getAPIServer() : optHost;
        final JSONObject response = RESTUtility.requestJsonRpc(host, configuration.getAPIKey(), "provisioning.deleteGuestUser", params, session).toObject();

        checkForError(response);
    }

    /**
     * Sends the specified identity token.
     *
     * <pre>
     * {
     *     "jsonrpc": "2.0",
     *     "method": "provisioning.storeToken",
     *     "params": {
     *         "apiKey": "API_KEY_01",
     *         "applicationUserId": 2,
     *         "applicationTenantId": 1,
     *         "applicationSessionId": 1,
     *         "token": "weqoilkjwaeo2jpdo9u2pokq2dpd"
     *     },
     *     "id": 100
     * }
     * </pre>
     *
     * @param userId The user identifier
     * @param contextId The context identifier
     * @param sessionId The session identifier
     * @param token The identity token
     * @param optHost The optional host
     * @throws MessengerException yet to be defined
     */
    public void sendToken(final int userId, final int contextId, final String sessionId, final UUID token, final String optHost) throws MessengerException {
        assertAuthenticated();

        final Map<String, Object> params = new LinkedHashMap<String, Object>(8);
        params.put("applicationUserId", Integer.valueOf(userId));
        params.put("applicationTenantId", Integer.valueOf(contextId));
        params.put("applicationSessionId", sessionId);
        params.put("token", UUIDs.getUnformattedString(token));

        final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
        final JSONObject response =
            RESTUtility.requestJsonRpc(Strings.isEmpty(optHost) ? configuration.getAPIServer() : optHost, configuration.getAPIKey(), "provisioning.storeToken", params, session).toObject();

        checkForError(response);
    }

    /**
     * Deletes given context.
     *
     * @param context The contact
     * @throws MessengerException If deletion fails
     */
    public void deleteContext(final Context context) throws MessengerException {
        deleteContext(context.getContextId());
    }

    /**
     * Deletes given context.
     *
     * @param contextId The contact identifier
     * @throws MessengerException If deletion fails
     */
    public void deleteContext(final int contextId) throws MessengerException {
        assertAuthenticated();

        final Map<String, Object> params = new LinkedHashMap<String, Object>(5);
        params.put("applicationTenantId", Integer.valueOf(contextId));

        final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
        final JSONValue jResponse = RESTUtility.requestJsonRpc(configuration.getAPIServer(), configuration.getAPIKey(), "provisioning.deleteTenant", params, session);
        checkForError(jResponse);
    }

    /**
     * Creates given user
     *
     * @param context The context
     * @param user The user
     * @param contact The user's contact
     * @return The JSON data for created user
     * @throws MessengerException If creation fails
     */
    public JSONValue createUser(final Context context, final com.openexchange.groupware.ldap.User user, final Contact contact) throws MessengerException {
        return createUser(context, user, contact, null);
    }

    /**
     * Creates given user
     *
     * @param context The context
     * @param user The user
     * @param contact The user's contact
     * @param mailAddress The optional mail address
     * @return The JSON data for created user
     * @throws MessengerException If creation fails
     */
    public JSONValue createUser(final Context context, final com.openexchange.groupware.ldap.User user, final Contact contact, final String mailAddress) throws MessengerException {
        return createUser(context.getContextId(), user.getId(), null == mailAddress ? user.getMail() : mailAddress, contact.getGivenName(), contact.getSurName());
    }

    /**
     * Creates given user
     *
     * @param contextId The context identifier
     * @param userId The user identifier
     * @param mailAddress The optional mail address
     * @param firstName The first name
     * @param lastName The last name
     * @return The JSON data for created user
     * @throws MessengerException If creation fails
     */
    public JSONValue createUser(final int contextId, final int userId, final String mailAddress, final String firstName, final String lastName) throws MessengerException {
        assertAuthenticated();

        try {
            final Map<String, Object> params = new LinkedHashMap<String, Object>(8);
            params.put("applicationUserId", Integer.valueOf(userId));
            params.put("applicationTenantId", Integer.valueOf(contextId));
            params.put("firstName", firstName);
            params.put("lastName", lastName);
            params.put("userName", mailAddress);

            final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            final JSONValue jResponse = RESTUtility.requestJsonRpc(configuration.getAPIServer(), configuration.getAPIKey(), "provisioning.createUser", params, session);
            checkForError(jResponse);

            return (JSONValue) jResponse.toObject().get("result");
        } catch (final JSONException e) {
            throw new MessengerException(e.getMessage(), e);
        }
    }

    /**
     * Deletes given user
     *
     * @param context
     * @param user The user
     * @param contact The user's contact
     * @throws MessengerException If deletion fails
     */
    public void deleteUser(final Context context, final com.openexchange.groupware.ldap.User user, final Contact contact) throws MessengerException {
        deleteUser(context.getContextId(), user.getId());
    }

    /**
     * Deletes given user
     *
     * @param contextId The context identifier
     * @param userId The user identifier
     * @throws MessengerException If deletion fails
     */
    public void deleteUser(final int contextId, final int userId) throws MessengerException {
        assertAuthenticated();

        final Map<String, Object> params = new LinkedHashMap<String, Object>(5);
        params.put("applicationUserId", Integer.valueOf(userId));
        params.put("applicationTenantId", Integer.valueOf(contextId));

        final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
        final JSONValue jResponse = RESTUtility.requestJsonRpc(configuration.getAPIServer(), configuration.getAPIKey(), "provisioning.deleteUser", params, session);
        checkForError(jResponse);
    }

    private void checkForError(final JSONValue jRes) throws MessengerApiException {
        if (jRes.isObject()) {
            final JSONObject jResponse = jRes.toObject();
            final JSONObject jError = jResponse.optJSONObject("error");
            if (null != jError) {
                throw new MessengerApiException(jError);
            }
        }
    }

}
