/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2020 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger;

import com.openexchange.config.ConfigTools;
import com.openexchange.config.ConfigurationService;
import com.openexchange.java.Strings;

/**
 * {@link MessengerConfiguration}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class MessengerConfiguration {

    private static final MessengerConfiguration INSTANCE = new MessengerConfiguration();

    /**
     * Gets the instance.
     *
     * @return The instance
     */
    public static MessengerConfiguration getInstance() {
        return INSTANCE;
    }

    // ------------------------------------------------------------------------ //

    private String apiServer;
    private String apiKey;
    private String userName;
    private String password;
    private boolean guestModeEnabled;
    private int guestModeTTLSeconds;
    private String guestModeCallbackUrl;
    private String proxyHostname;
    private int proxyPort = Integer.MIN_VALUE;
    private String proxyUsername;
    private String proxyPassword;
    private boolean secureSsl;

    /**
     * Initializes a new {@link MessengerConfiguration}.
     */
    private MessengerConfiguration() {
        super();
    }

    /**
     * Reads OX Messenger configuration from given configuration service.
     *
     * @param configService The configuration service
     */
    public void readFrom(final ConfigurationService configService) {
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.apiServer");
            apiServer = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.apiKey");
            apiKey = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.userName");
            userName = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.password");
            password = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            guestModeEnabled = configService.getBoolProperty("com.openexchange.messenger.guestMode.enabled", true);
        }
        {
            guestModeTTLSeconds = ConfigTools.parseTimespanSecs(configService.getProperty("com.openexchange.messenger.guestMode.ttl", "5D"));
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.guestMode.url");
            guestModeCallbackUrl = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.proxy.hostname");
            proxyHostname = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.proxy.port");
            if (! Strings.isEmpty(tmp)) {
                proxyPort = Integer.parseInt(tmp.trim());
            } else {
                proxyPort = Integer.MIN_VALUE;
            }
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.proxy.username");
            proxyUsername = Strings.isEmpty(tmp) ? null : tmp.trim();
        }
        {
            proxyPassword = configService.getProperty("com.openexchange.messenger.proxy.password");
        }
        {
            final String tmp = configService.getProperty("com.openexchange.messenger.secure.ssl");
            secureSsl = Strings.isEmpty(tmp) || (! tmp.trim().toLowerCase().equals("false"));
        }
    }

    /**
     * Gets the API server
     *
     * @return The server name
     */
    public String getAPIServer() {
        return apiServer;
    }

    /**
     * Gets the API key
     *
     * @return The API key
     */
    public String getAPIKey() {
        return apiKey;
    }

    /**
     * Gets the user name.
     *
     * @return The user name
     */
    public String getUserName() {
        return userName;
    }

    /**
     * Gets the password
     *
     * @return The password
     */
    public String getPassword() {
        return password;
    }

    /**
     * Checks if guest mode is globally enabled/disabled as per configuration
     *
     * @return <code>true</code> if enabled; otherwise <code>false</code>
     */
    public boolean isGuestModeEnabled() {
        return guestModeEnabled;
    }

    /**
     * Gets the time-to-live seconds that applies to a sent invitation. After that time span the invotation/guest session expires.
     *
     * @return The time-to-live seconds
     */
    public int getGuestModeTTLSeconds() {
        return guestModeTTLSeconds;
    }


    /**
     * Gets the call-back URL for invited guest users.
     *
     * @return The call-back URL or <code>null</code> which signals to use calling node's URL
     */
    public String getGuestModeCallbackUrl() {
        return guestModeCallbackUrl;
    }

    /**
     * Gets the proxy hostname, or {@code null} if no proxy hostname is defined.
     * @return the proxy hostname or {@code null} if no proxy hostname is defined
     */
    public String getProxyHostname() {
        return proxyHostname;
    }

    /**
     * Gets the proxy port, or {@link Integer#MIN_VALUE} if no proxy port is defined.
     * @return the proxy port, or {@link Integer#MIN_VALUE} if no proxy port is defined
     */
    public int getProxyPort() {
        return proxyPort;
    }

    /**
     * Gets the proxy authentication username, or {@code null} if no proxy username is defined.
     * @return the proxy authentication username, or {@code null} if no proxy username is defined
     */
    public String getProxyUsername() {
        return proxyUsername;
    }

    /**
     * Gets the proxy authentication password, or {@code null} if no proxy password is defined.
     * @return the proxy authentication password, or {@code null} if no proxy password is defined
     */
    public String getProxyPassword() {
        return proxyPassword;
    }

    /**
     * Whether to use secure SSL or not.
     * @return whether to use secure SSL or not
     */
    public boolean isSecureSsl() {
        return secureSsl;
    }
}
