/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger.mbean.impl;

import javax.management.MBeanException;
import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;
import org.slf4j.Logger;
import com.openexchange.auth.Authenticator;
import com.openexchange.auth.Credentials;
import com.openexchange.context.ContextService;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.ldap.User;
import com.openexchange.messenger.MessengerAPI;
import com.openexchange.messenger.MessengerConfiguration;
import com.openexchange.messenger.mbean.MessengerMBean;
import com.openexchange.messenger.osgi.Services;
import com.openexchange.messenger.session.StaticAuthSession;
import com.openexchange.server.ServiceExceptionCode;
import com.openexchange.user.UserService;


/**
 * {@link MessengerMBeanImpl}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class MessengerMBeanImpl extends StandardMBean implements MessengerMBean {

    /**
     * Initializes a new {@link MessengerMBeanImpl}.
     *
     * @throws NotCompliantMBeanException If initialization fails
     */
    public MessengerMBeanImpl() throws NotCompliantMBeanException {
        super(MessengerMBean.class);
    }

    @Override
    public void createUser(int userId, int contextId, String mailAddress, String firstName, String lastName, String adminLogin, String adminPassword) throws MBeanException {
        Logger logger = org.slf4j.LoggerFactory.getLogger(MessengerMBeanImpl.class);
        try {
            Authenticator authenticator = Services.getService(Authenticator.class);
            if (null == authenticator) {
                throw ServiceExceptionCode.absentService(Authenticator.class);
            }
            authenticator.doAuthentication(new Credentials(adminLogin, adminPassword));

            MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            MessengerAPI<StaticAuthSession> api = new MessengerAPI<StaticAuthSession>(new StaticAuthSession(configuration));
            api.createUser(contextId, userId, mailAddress, firstName, lastName);
        } catch (Exception e) {
            logger.error("", e);
            String message = e.getMessage();
            throw new MBeanException(new Exception(message), message);
        }
    }

    @Override
    public void deleteUser(int userId, int contextId, String adminLogin, String adminPassword) throws MBeanException {
        Logger logger = org.slf4j.LoggerFactory.getLogger(MessengerMBeanImpl.class);
        try {
            Authenticator authenticator = Services.getService(Authenticator.class);
            if (null == authenticator) {
                throw ServiceExceptionCode.absentService(Authenticator.class);
            }
            authenticator.doAuthentication(new Credentials(adminLogin, adminPassword));

            MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            MessengerAPI<StaticAuthSession> api = new MessengerAPI<StaticAuthSession>(new StaticAuthSession(configuration));
            api.deleteUser(contextId, userId);
        } catch (Exception e) {
            logger.error("", e);
            String message = e.getMessage();
            throw new MBeanException(new Exception(message), message);
        }
    }

    @Override
    public void createContext(int contextId, boolean full, String adminLogin, String adminPassword) throws MBeanException {
        Logger logger = org.slf4j.LoggerFactory.getLogger(MessengerMBeanImpl.class);
        try {
            Authenticator authenticator = Services.getService(Authenticator.class);
            if (null == authenticator) {
                throw ServiceExceptionCode.absentService(Authenticator.class);
            }
            authenticator.doAuthentication(new Credentials(adminLogin, adminPassword));

            ContextService contextService = Services.getService(ContextService.class);
            if (null == contextService) {
                throw ServiceExceptionCode.absentService(ContextService.class);
            }

            UserService userService = Services.getService(UserService.class);
            if (null == userService) {
                throw ServiceExceptionCode.absentService(UserService.class);
            }

            MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            MessengerAPI<StaticAuthSession> api = new MessengerAPI<StaticAuthSession>(new StaticAuthSession(configuration));

            Context context = contextService.getContext(contextId);
            if (full) {
                for (int userId : userService.listAllUser(context)) {
                    User user = userService.getUser(userId, context);
                    api.createUser(contextId, userId, user.getMail(), user.getGivenName(), user.getSurname());
                }
            } else {
                int adminId = context.getMailadmin();
                User user = userService.getUser(adminId, context);
                api.createUser(contextId, adminId, user.getMail(), user.getGivenName(), user.getSurname());
            }
        } catch (Exception e) {
            logger.error("", e);
            String message = e.getMessage();
            throw new MBeanException(new Exception(message), message);
        }
    }

    @Override
    public void deleteContext(int contextId, String adminLogin, String adminPassword) throws MBeanException {
        Logger logger = org.slf4j.LoggerFactory.getLogger(MessengerMBeanImpl.class);
        try {
            Authenticator authenticator = Services.getService(Authenticator.class);
            if (null == authenticator) {
                throw ServiceExceptionCode.absentService(Authenticator.class);
            }
            authenticator.doAuthentication(new Credentials(adminLogin, adminPassword));

            MessengerConfiguration configuration = MessengerConfiguration.getInstance();
            MessengerAPI<StaticAuthSession> api = new MessengerAPI<StaticAuthSession>(new StaticAuthSession(configuration));
            api.deleteContext(contextId);
        } catch (Exception e) {
            logger.error("", e);
            String message = e.getMessage();
            throw new MBeanException(new Exception(message), message);
        }
    }

}
