/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger.osgi;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;
import javax.management.ObjectName;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.auth.Authenticator;
import com.openexchange.capabilities.CapabilityService;
import com.openexchange.config.ConfigurationService;
import com.openexchange.context.ContextService;
import com.openexchange.exception.OXException;
import com.openexchange.jump.Endpoint;
import com.openexchange.jump.EndpointHandler;
import com.openexchange.management.ManagementService;
import com.openexchange.management.Managements;
import com.openexchange.messenger.MessengerAPI;
import com.openexchange.messenger.MessengerConfiguration;
import com.openexchange.messenger.MessengerExceptionCodes;
import com.openexchange.messenger.exception.MessengerException;
import com.openexchange.messenger.mbean.MessengerMBean;
import com.openexchange.messenger.mbean.impl.MessengerMBeanImpl;
import com.openexchange.messenger.session.StaticAuthSession;
import com.openexchange.osgi.HousekeepingActivator;
import com.openexchange.osgi.SimpleRegistryListener;
import com.openexchange.session.Session;
import com.openexchange.threadpool.ThreadPoolService;
import com.openexchange.timer.TimerService;
import com.openexchange.user.UserService;

/**
 * {@link MessengerActivator}
 *
 * @author <a href="mailto:ioannis.chouklis@open-xchange.com">Ioannis Chouklis</a>
 */
public class MessengerActivator extends HousekeepingActivator {

    @Override
    protected Class<?>[] getNeededServices() {
        return new Class<?>[] { ConfigurationService.class, TimerService.class, ThreadPoolService.class, ContextService.class, UserService.class };
    }

    @Override
    protected void startBundle() throws Exception {
        final Logger logger = LoggerFactory.getLogger(MessengerActivator.class);
        logger.info("starting bundle: {}", context.getBundle().getSymbolicName());

        // Set service look-up
        Services.setServiceLookup(this);

        // Tracker
        track(ManagementService.class, new SimpleRegistryListener<ManagementService>() {

            @Override
            public void added(final ServiceReference<ManagementService> ref, final ManagementService management) {
                try {
                    final ObjectName objectName = Managements.getObjectName(MessengerMBean.class.getName(), MessengerMBean.DOMAIN);
                    management.registerMBean(objectName, new MessengerMBeanImpl());
                } catch (final Exception e) {
                    logger.warn("Could not register MBean {}", MessengerMBean.class.getName());
                }
            }

            @Override
            public void removed(final ServiceReference<ManagementService> ref, final ManagementService management) {
                try {
                    management.unregisterMBean(Managements.getObjectName(MessengerMBean.class.getName(), MessengerMBean.DOMAIN));
                } catch (final Exception e) {
                    logger.warn("Could not un-register MBean {}", MessengerMBean.class.getName());
                }
            }
        });
        trackService(Authenticator.class);
        trackService(CapabilityService.class);
        openTrackers();

        // Register handler
        {
            final EndpointHandler handler = new EndpointHandler() {

                private final Set<String> namesOfInterest = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList("messenger")));

                @Override
                public Set<String> systemNamesOfInterest() {
                    return namesOfInterest;
                }

                @Override
                public boolean handleEndpoint(final UUID token, final Endpoint endpoint, final Session session) throws OXException {
                    // pass to end-point
                    final MessengerConfiguration configuration = MessengerConfiguration.getInstance();
                    final MessengerAPI<StaticAuthSession> api = new MessengerAPI<StaticAuthSession>(new StaticAuthSession(configuration));
                    try {
                        api.sendToken(session.getUserId(), session.getContextId(), session.getSessionID(), token, endpoint.getUrl());
                        return true;
                    } catch (final MessengerException e) {
                        throw MessengerExceptionCodes.MESSENGER_ERROR.create(e, e.getMessage());
                    } catch (final RuntimeException e) {
                        throw MessengerExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
                    }
                }
            };
            registerService(EndpointHandler.class, handler);
        }

        // Initialize configuration
        final ConfigurationService cs = getService(ConfigurationService.class);
        MessengerConfiguration.getInstance().readFrom(cs);
    }

    @Override
    protected void stopBundle() throws Exception {
        Services.setServiceLookup(null);
        super.stopBundle();
    }
}
