/*-
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.messenger.rest;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PushbackReader;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Scanner;
import java.util.concurrent.atomic.AtomicLong;
import javax.net.ssl.SSLException;
import org.apache.commons.codec.CharEncoding;
import org.apache.commons.codec.EncoderException;
import org.apache.commons.codec.net.URLCodec;
import org.apache.http.HttpEntity;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.AbstractHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.json.JSONException;
import org.json.JSONInputStream;
import org.json.JSONObject;
import org.json.JSONValue;
import com.openexchange.java.Streams;
import com.openexchange.java.Strings;
import com.openexchange.messenger.MessengerAPI;
import com.openexchange.messenger.exception.MessengerApiException;
import com.openexchange.messenger.exception.MessengerException;
import com.openexchange.messenger.exception.MessengerIOException;
import com.openexchange.messenger.exception.MessengerParseException;
import com.openexchange.messenger.exception.MessengerSSLException;
import com.openexchange.messenger.exception.MessengerServerException;
import com.openexchange.messenger.exception.MessengerUnlinkedException;
import com.openexchange.messenger.httpclient.PreemptiveBasicProxyAuthenticationInterceptor;
import com.openexchange.messenger.session.Session;
import com.openexchange.messenger.session.Session.ProxyInfo;

/**
 * This class is mostly used internally by {@link MessengerAPI} for creating and executing REST requests to the OX Messenger API, and parsing
 * responses. You probably won't have a use for it other than {@link #parseDate(String)} for parsing modified times returned in metadata, or
 * (in very rare circumstances) writing your own API calls.
 */
public class RESTUtility {

    /**
     * No initialization.
     */
    private RESTUtility() {
        super();
    }

    private static final AtomicLong JSONRPC_COUNTER = new AtomicLong();

    private static final DateFormat dateFormat = new SimpleDateFormat("EEE, dd MMM yyyy kk:mm:ss ZZZZZ", Locale.US);

    /** The HTTP method enumeration */
    public static enum Method {
        GET, POST, PUT, DELETE;
    }

    /**
     * Creates and sends a request to the OX Messenger API, parses the response as JSON, and returns the result.
     *
     * @param method GET or POST.
     * @param host the hostname to use. Should be either api server, content server, or web server.
     * @param path the URL path, starting with a '/'.
     * @param apiVersion the API version to use.
     * @param session the {@link Session} to use for this request.
     * @return a parsed JSON object, typically a Map or a JSONArray.
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerParseException if a malformed or unknown response was received from the server.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static JSONValue request(final Method method, final String host, final String path, final int apiVersion, final Session session) throws MessengerException {
        final HttpResponse resp = streamRequest(method, host, path, apiVersion, null, session).response;
        return parseAsJSON(resp);
    }

    /**
     * Creates and sends a request to the OX Messenger API, parses the response as JSON, and returns the result.
     *
     * @param method GET or POST.
     * @param host the hostname to use. Should be either api server, content server, or web server.
     * @param path the URL path, starting with a '/'.
     * @param apiVersion the API version to use.
     * @param params the URL params in an array, with the even numbered elements the parameter names and odd numbered elements the values,
     *            e.g. <code>new String[] {"path", "/Public", "locale",
     *         "en"}</code>.
     * @param session the {@link Session} to use for this request.
     * @return a parsed JSON object, typically a Map or a JSONArray.
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerParseException if a malformed or unknown response was received from the server.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static JSONValue request(final Method method, final String host, final String path, final int apiVersion, final String[] params, final Session session) throws MessengerException {
        final HttpResponse resp = streamRequest(method, host, path, apiVersion, params, session).response;
        return parseAsJSON(resp);
    }

    /**
     * Executes a JSONRPC request.
     *
     * @param host The host name
     * @param apiKey The API key
     * @param methodName The method name; e.g. <code>"provisioning.createUser"</code>
     * @param params The parameters; e.g. <code>"applicationUserId"</code>, <code>"applicationTenantId"</code> or
     *            <code>"applicationSessionId"</code>
     * @param session The associated session
     * @return The resulting JSON response
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerParseException if a malformed or unknown response was received from the server.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static JSONValue requestJsonRpc(final String host, final String apiKey, final String methodName, final Map<String, Object> params, final Session session) throws MessengerException {
        try {
            final JSONObject jsonRpcRequest = new JSONObject(6);

            // Put basic information
            jsonRpcRequest.put("jsonrpc", "2.0").put("method", methodName);

            // Add parameters
            {
                final JSONObject jParams = new JSONObject(8);
                jParams.put("apiKey", apiKey);
                if (params != null && !params.isEmpty()) {
                    for (final Entry<String, Object> entry : params.entrySet()) {
                        jParams.put(entry.getKey(), entry.getValue());
                    }
                }
                jsonRpcRequest.put("params", jParams);
            }

            jsonRpcRequest.put("id", JSONRPC_COUNTER.incrementAndGet());

            // Fire request
            final HttpResponse resp = streamRequest(Method.POST, host, null, 0, null, jsonRpcRequest, session).response;
            return parseAsJSON(resp, false);
        } catch (final JSONException e) {
            throw new MessengerException(e);
        }
    }

    /**
     * Creates and sends a request to the OX Messenger API, and returns a {@link RequestAndResponse} containing the {@link HttpUriRequest}
     * and {@link HttpResponse}.
     *
     * @param method GET or POST.
     * @param host the hostname to use. Should be either api server, content server, or web server.
     * @param path the URL path, starting with a '/'.
     * @param apiVersion the API version to use.
     * @param params the URL params in an array, with the even numbered elements the parameter names and odd numbered elements the values,
     *            e.g. <code>new String[] {"path", "/Public", "locale",
     *         "en"}</code>.
     * @param session the {@link Session} to use for this request.
     * @return a parsed JSON object, typically a Map or a JSONArray.
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static RequestAndResponse streamRequest(final Method method, final String host, final String path, final int apiVersion, final String[] params, final Session session) throws MessengerException {
        return streamRequest(method, host, path, apiVersion, params, null, session);
    }

    /**
     * Creates and sends a request to the OX Messenger API, and returns a {@link RequestAndResponse} containing the {@link HttpUriRequest}
     * and {@link HttpResponse}.
     *
     * @param method GET or POST.
     * @param host the hostname to use. Should be either api server, content server, or web server.
     * @param path the URL path, starting with a '/'.
     * @param optApiVersion the API version to use. Or <code>-1</code> to ignore.
     * @param params the URL params in an array, with the even numbered elements the parameter names and odd numbered elements the values,
     *            e.g. <code>new String[] {"path", "/Public", "locale",
     *         "en"}</code>.
     * @param requestInformation The request's JSON object
     * @param session the {@link Session} to use for this request.
     * @return a parsed JSON object, typically a Map or a JSONArray.
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static RequestAndResponse streamRequest(final Method method, final String host, final String path, final int optApiVersion, final String[] params, final JSONObject requestInformation, final Session session) throws MessengerException {
        final HttpRequestBase req;
        switch (method) {
        case PUT: {
            final HttpPut put = new HttpPut(buildURL(host, optApiVersion, path, params));
            if (null != requestInformation) {
                put.setEntity(new InputStreamEntity(new JSONInputStream(requestInformation, "UTF-8"), -1L, ContentType.APPLICATION_JSON));
            }
            req = put;
        }
            break;
        case POST: {
            final HttpPost post = new HttpPost(buildURL(host, optApiVersion, path, params));
            if (null != requestInformation) {
                post.setEntity(new InputStreamEntity(new JSONInputStream(requestInformation, "UTF-8"), -1L, ContentType.APPLICATION_JSON));
            }
            req = post;
        }
            break;
        case GET:
            req = new HttpGet(buildURL(host, optApiVersion, path, params));
            break;
        case DELETE:
            req = new HttpDelete(buildURL(host, optApiVersion, path, params));
            break;
        default:
            throw new MessengerException("Unsupported HTTP method: " + method);
        }
        // Sign request
        session.sign(req);
        final HttpResponse resp = execute(session, req);
        return new RequestAndResponse(req, resp);
    }

    /**
     * Reads in content from an {@link HttpResponse} and parses it as JSON.
     *
     * @param response the {@link HttpResponse}.
     * @return a parsed JSON object, typically a Map or a JSONArray.
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs while reading in content from the {@link HttpResponse}.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerParseException if a mal-formed or unknown response was received from the server.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static JSONValue parseAsJSON(final HttpResponse response) throws MessengerException {
        return parseAsJSON(response, true);
    }

    /**
     * Reads in content from an {@link HttpResponse} and parses it as JSON.
     *
     * @param response the {@link HttpResponse}.
     * @param failIfEmpty <code>true</code> to fail on empty response; otherwise <code>false</code>
     * @return A parsed JSON object, typically a Map or a JSONArray; or <code>null</code> on empty response and if <code>failIfEmpty</code>
     *         is <code>false</code>
     * @throws MessengerServerException if the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException if any network-related error occurs while reading in content from the {@link HttpResponse}.
     * @throws MessengerUnlinkedException if the user has revoked access.
     * @throws MessengerParseException if a mal-formed or unknown response was received from the server.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static JSONValue parseAsJSON(final HttpResponse response, final boolean failIfEmpty) throws MessengerException {
        JSONValue result = null;

        BufferedReader bin = null;
        try {
            final HttpEntity ent = response.getEntity();
            if (ent != null) {
                final PushbackReader pir = new PushbackReader(new InputStreamReader(ent.getContent()));
                final int read = pir.read();
                if (read < 0) {
                    if (failIfEmpty) {
                        throw new MessengerServerException(response, result);
                    }
                    return null;
                }
                pir.unread(read);

                // Wrap this with a Buffer, so we can re-parse it if it's
                // not JSON
                // Has to be at least 16384, because this is defined as the buffer size in
                // org.json.simple.parser.Yylex.java
                // and otherwise the reset() call won't work
                bin = new BufferedReader(pir, 16384);
                bin.mark(16384);

                result = JSONObject.parse(bin);
                if (result.isObject()) {
                    checkForError(result.toObject());
                }
            }
        } catch (final IOException e) {
            throw new MessengerIOException(e);
        } catch (final JSONException e) {
            if (MessengerServerException.isValidWithNullBody(response)) {
                // We have something, but it's an error with no reason
                throw new MessengerServerException(response);
            }
            // This is from messenger, and we shouldn't be getting it
            throw new MessengerParseException(bin);
        } catch (final OutOfMemoryError e) {
            throw new MessengerException(e);
        } finally {
            Streams.close(bin);
        }

        final int statusCode = response.getStatusLine().getStatusCode();
        if (statusCode != MessengerServerException._200_OK) {
            if (statusCode == MessengerServerException._401_UNAUTHORIZED) {
                throw new MessengerUnlinkedException();
            }
            throw new MessengerServerException(response, (result != null) ? result : Integer.toString(statusCode));
        }

        return result;
    }

    private static void checkForError(final JSONObject responseObject) throws MessengerApiException {
        if (responseObject.has("error")) {
            throw new MessengerApiException(responseObject.optJSONObject("error"));
        }
    }

    /**
     * Reads in content from an {@link HttpResponse} and parses it as a query string.
     *
     * @param response The {@link HttpResponse}.
     * @return A map of parameter names to values from the query string.
     * @throws MessengerIOException if any network-related error occurs while reading in content from the {@link HttpResponse}.
     * @throws MessengerParseException if a malformed or unknown response was received from the server.
     * @throws MessengerException for any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static Map<String, String> parseAsQueryString(final HttpResponse response) throws MessengerException {
        final HttpEntity entity = response.getEntity();
        if (entity == null) {
            throw new MessengerParseException("Bad response from messenger.");
        }

        final Scanner scanner;
        try {
            scanner = new Scanner(entity.getContent()).useDelimiter("&");
        } catch (final IOException e) {
            throw new MessengerIOException(e);
        }

        final Map<String, String> result = new HashMap<String, String>();
        while (scanner.hasNext()) {
            final String nameValue = scanner.next();
            final String[] parts = nameValue.split("=");
            if (parts.length != 2) {
                throw new MessengerParseException("Bad query string from messenger.");
            }
            result.put(parts[0], parts[1]);
        }
        return result;
    }

    /**
     * Executes an {@link HttpUriRequest} with the given {@link Session} and returns an {@link HttpResponse}.
     *
     * @param session The session to use.
     * @param req The request to execute.
     * @return An {@link HttpResponse}.
     * @throws MessengerServerException If the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException If any network-related error occurs.
     * @throws MessengerUnlinkedException If the user has revoked access.
     * @throws MessengerException For any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static HttpResponse execute(final Session session, final HttpUriRequest req) throws MessengerException {
        return execute(session, req, -1);
    }

    /**
     * Executes an {@link HttpUriRequest} with the given {@link Session} and returns an {@link HttpResponse}.
     *
     * @param session The session to use.
     * @param req The request to execute.
     * @param socketTimeoutOverrideMs If >= 0, the socket timeout to set on this request. Does nothing if set to a negative number.
     * @return An {@link HttpResponse}.
     * @throws MessengerServerException If the server responds with an error code. See the constants in {@link MessengerServerException} for the meaning
     *             of each error code.
     * @throws MessengerIOException If any network-related error occurs.
     * @throws MessengerUnlinkedException If the user has revoked access.
     * @throws MessengerException For any other unknown errors. This is also a superclass of all other OX Messenger exceptions, so you may want to
     *             only catch this exception which signals that some kind of error occurred.
     */
    public static HttpResponse execute(final Session session, final HttpUriRequest req, final int socketTimeoutOverrideMs) throws MessengerException {
        final HttpClient client = updatedHttpClient(session);

        // Set request timeouts.
        session.setRequestTimeout(req);
        if (socketTimeoutOverrideMs >= 0) {
            final HttpParams reqParams = req.getParams();
            HttpConnectionParams.setSoTimeout(reqParams, socketTimeoutOverrideMs);
        }

        final boolean repeatable = isRequestRepeatable(req);

        try {
            HttpResponse response = null;
            for (int retries = 0; response == null && retries < 5; retries++) {
                /*
                 * The try/catch is a workaround for a bug in the HttpClient libraries. It should be returning null instead when an error
                 * occurs. Fixed in HttpClient 4.1, but we're stuck with this for now. See:
                 * http://code.google.com/p/android/issues/detail?id=5255
                 */
                try {
                    response = client.execute(req);
                } catch (final NullPointerException e) {
                    // Leave 'response' as null. This is handled below.
                    e.printStackTrace();
                }

                /*
                 * We've potentially connected to a different network, but are still using the old proxy settings. Refresh proxy settings so
                 * that we can retry this request.
                 */
                if (response == null) {
                    updateClientProxy(client, session);
                }

                if (!repeatable) {
                    break;
                }
            }

            if (response == null) {
                // This is from that bug, and retrying hasn't fixed it.
                throw new MessengerIOException("Apache HTTPClient encountered an error. No response, try again.");
            }

            final int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != MessengerServerException._200_OK && statusCode != MessengerServerException._206_PARTIAL_CONTENT) {
                parseAsJSON(response);
            }

            return response;
        } catch (final SSLException e) {
            throw new MessengerSSLException(e);
        } catch (final IOException e) {
            // Quite common for network going up & down or the request being
            // cancelled, so don't worry about logging this
            throw new MessengerIOException(e);
        } catch (final OutOfMemoryError e) {
            throw new MessengerException(e);
        }
    }

    private static boolean isRequestRepeatable(final HttpRequest req) {
        // If the request contains an HttpEntity that can't be "reset" (like an InputStream),
        // then it isn't repeatable.
        if (req instanceof HttpEntityEnclosingRequest) {
            final HttpEntityEnclosingRequest ereq = (HttpEntityEnclosingRequest) req;
            final HttpEntity entity = ereq.getEntity();
            if (entity != null && !entity.isRepeatable()) {
                return false;
            }
        }
        return true;
    }

    /**
     * Creates a URL for a request to the OX Messenger API.
     *
     * @param host The OX Messenger host (i.e., api server, content server, or web server).
     * @param optApiVersion The API version to use. Or <code>-1</code> to ignore
     * @param target The target path, staring with a '/'.
     * @param params Any URL params in an array, with the even numbered elements the parameter names and odd numbered elements the values,
     *            e.g. <code>new String[] {"path", "/Public", "locale", "en"}</code>.
     * @return A full URL for making a request.
     */
    public static String buildURL(final String host, final int optApiVersion, final String target, final String[] params) {
        if (Strings.isEmpty(target)) {
            String trgt = "";
            if (params != null && params.length > 0) {
                final StringBuilder sb = new StringBuilder(params.length << 4);
                sb.append('?').append(urlencode(params));
                trgt = sb.toString();
            }

            final StringBuilder sb = new StringBuilder(64);
            if (host.startsWith("http://") || host.startsWith("https://")) {
                sb.append(host);
            } else {
                sb.append("https://").append(host).append(":443");
            }

            return sb.append(trgt).toString();
        }

        // Path is not empty
        String trgt = target;
        if (!trgt.startsWith("/")) {
            trgt = new StringBuilder(trgt.length() + 1).append('/').append(trgt).toString();
        }

        try {
            // We have to encode the whole line, then remove + and / encoding
            // to get a good OAuth URL.
            if (optApiVersion > 0) {
                trgt = URLEncoder.encode(new StringBuilder(16).append("/v").append(optApiVersion).append(trgt).toString(), "UTF-8");
            } else {
                trgt = URLEncoder.encode(new StringBuilder(16).append(trgt).toString(), "UTF-8");
            }
            trgt = trgt.replace("%2F", "/");

            if (params != null && params.length > 0) {
                final StringBuilder sb = new StringBuilder(trgt);
                sb.append('?').append(urlencode(params));
                trgt = sb.toString();
            }

            // These substitutions must be made to keep OAuth happy.
            trgt = trgt.replace("+", "%20").replace("*", "%2A");
        } catch (final UnsupportedEncodingException uce) {
            return null;
        }

        final StringBuilder sb = new StringBuilder(64);
        if (host.startsWith("http://") || host.startsWith("https://")) {
            sb.append(host);
        } else {
            sb.append("https://").append(host).append(":443");
        }

        return sb.append(trgt).toString();
    }

    /**
     * Parses a date/time returned by the OX Messenger API. Returns <code>null</code> if it cannot be parsed.
     *
     * @param date A date returned by the API.
     * @return A {@link Date}.
     */
    public static Date parseDate(final String date) {
        try {
            return dateFormat.parse(date);
        } catch (final java.text.ParseException e) {
            return null;
        }
    }

    /**
     * Gets the session's client and updates its proxy.
     */
    private static synchronized HttpClient updatedHttpClient(final Session session) {
        final HttpClient client = session.getHttpClient();
        updateClientProxy(client, session);
        return client;
    }

    /**
     * Updates the given client's proxy from the session.
     */
    private static void updateClientProxy(final HttpClient client, final Session session) {
        final ProxyInfo proxyInfo = session.getProxyInfo();
        if (proxyInfo != null && proxyInfo.host != null && !proxyInfo.host.equals("")) {
            HttpHost proxy;
            if (proxyInfo.port < 0) {
                proxy = new HttpHost(proxyInfo.host);
            } else {
                proxy = new HttpHost(proxyInfo.host, proxyInfo.port);
            }
            client.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);
            {
                final String username = proxyInfo.username;
                final String password = proxyInfo.password;
                if (username != null && password != null && client instanceof AbstractHttpClient) {
                    PreemptiveBasicProxyAuthenticationInterceptor.register(username, password, (AbstractHttpClient) client);
                }
            }
        } else {
            client.getParams().removeParameter(ConnRoutePNames.DEFAULT_PROXY);
            if (client instanceof AbstractHttpClient) {
                PreemptiveBasicProxyAuthenticationInterceptor.remove((AbstractHttpClient) client);
            }
        }
    }

    /**
     * URL encodes an array of parameters into a query string.
     */
    private static String urlencode(final String[] params) {
        if (params.length % 2 != 0) {
            throw new IllegalArgumentException("Params must have an even number of elements.");
        }
        try {
            final StringBuilder result = new StringBuilder(params.length << 4);
            boolean firstTime = true;
            for (int i = 0; i < params.length; i += 2) {
                final String value = params[i + 1];
                if (null != value) {
                    if (firstTime) {
                        firstTime = false;
                    } else {
                        result.append('&');
                    }
                    result.append(encodeUrl(params[i])).append('=').append(encodeUrl(value));
                }
            }
            return result.toString().replace("*", "%2A");
        } catch (final RuntimeException e) {
            return null;
        }
    }

    private static final URLCodec URL_CODEC = new URLCodec(CharEncoding.UTF_8);

    /**
     * URL encodes given string.
     */
    public static final String encodeUrl(final String s) {
        try {
            return isEmpty(s) ? s : URL_CODEC.encode(s);
        } catch (final EncoderException e) {
            return s;
        }
    }

    private static boolean isEmpty(final String string) {
        if (null == string) {
            return true;
        }
        final int len = string.length();
        boolean isWhitespace = true;
        for (int i = 0; isWhitespace && i < len; i++) {
            isWhitespace = com.openexchange.java.Strings.isWhitespace(string.charAt(i));
        }
        return isWhitespace;
    }

}
